"use strict";

angular.module('siggo')
  .factory('$$flashlight', function($timeout, $cordovaFlashlight) {
    var isFlashing = false;

    var onFlash = function(time) {
      $cordovaFlashlight.available().then(
        function(isAvailable) {

          if (!isAvailable && isFlashing) {
            console.error("Flashlight not available on this device");
            turnOff(time);
            return;
          }
          $cordovaFlashlight.switchOn().then(
            function() {
              isFlashing = true;
              turnOff(time);
            },
            function(){
              console.error("Flashlight not available on this device");
              turnOff(0);
            }
          );

        },
        function() {
          console.error("Flashlight not available on this device");
        });
    }


    var patternManager = function(pattern) {
      var val = pattern.shift();
      var to = val + pattern.shift();

      onFlash(val);

      if (!to) return;
      $timeout(function() {
        patternManager(pattern);
      }, to);
    }


    var turnOff = function(time) {
      $timeout(function() {
        if (!isFlashing) return;
        $cordovaFlashlight.switchOff().then(
          function() {
            isFlashing = false;
          },
          function() {
            turnOff(100)
          });
      }, time);
    }



    return {
      isFlashing: isFlashing,

      flash: function(time) {
        if (time > 0) {
          onFlash(time);
        } else if (time instanceof Array) {
          patternManager(time);
        }
      },
      cancelFlash: function() {
        turnOff(0);
      }
    };
  });
