'use strict';

angular.module('siggo')
  .service('toaster', function ($rootScope) {

    var pop = function (data) {
      this.toast = data;
      $rootScope.$broadcast('toaster-newToast');
    };
    return {
      pop: pop
    }
  })
  .constant('toasterConfig', {
    'tap-to-dismiss': true,
    'newest-on-top': true,
    //'fade-in': 1000,            // done in css
    //'on-fade-in': undefined,    // not implemented
    //'fade-out': 1000,           // done in css
    // 'on-fade-out': undefined,  // not implemented
    //'extended-time-out': 1000,    // not implemented
    //'time-out': 5000, // Set timeOut and extendedTimeout to 0 to make it sticky
    'icon-classes': {
      error: 'toast-error',
      info: 'toast-info',
      success: 'toast-success',
      warning: 'toast-warning'
    },
    'icon-class': 'toast-info',
    'position-class': 'toast-top-full-width',
    'title-class': 'toast-title',
    'message-class': 'toast-message'
  })
  .directive('toasterContainer', ['$compile', '$timeout', 'toasterConfig', 'toaster',
    function ($compile, $timeout, toasterConfig, toaster) {
      return {
        replace: true,
        restrict: 'EA',
        link: function (scope, elm, attrs) {

          var id = 0;

          var mergedConfig = toasterConfig;
          if (attrs.toasterOptions) {
            angular.extend(mergedConfig, scope.$eval(attrs.toasterOptions));
          }

          scope.config = {
            position: mergedConfig['position-class'],
            title: mergedConfig['title-class'],
            message: mergedConfig['message-class'],
            tap: mergedConfig['tap-to-dismiss']
          };

          function addToast(toast) {
            toast.type = mergedConfig['icon-classes'][toast.type];
            if (!toast.type)
              toast.type = mergedConfig['icon-class'];

            id++;
            angular.extend(toast, {
              id: id
            });

            if (toast.timeout > 0)
              setTimeout(toast);

            if (mergedConfig['newest-on-top'] === true)
              scope.toasters.unshift(toast);
            else
              scope.toasters.push(toast);

            scope.$apply()
          }

          function setTimeout(toast) {
            toast.timeout = $timeout(function () {
              scope.removeToast(toast.id);
            }, toast.timeout);
          }

          scope.toasters = [];
          scope.$on('toaster-newToast', function () {
            addToast(toaster.toast);
          });
        },
        controller: function ($scope, $element, $attrs) {

          $scope.removeToast = function (id) {
            console.log(id);
            var i = 0;
            for (i; i < $scope.toasters.length; i++) {
              if ($scope.toasters[i].id === id)
                break;
            }
            $scope.toasters.splice(i, 1);
          };

          $scope.remove = function (id) {
            if ($scope.config.tap === true) {
              $scope.removeToast(id);
            }
          };
        },
        template: '<div id="toast-container" ng-class="config.position">' +
          '<div ng-animate="\'animateToaster\'" ng-repeat="toaster in toasters">' +
          '<div class="toast" ng-class="toaster.type" ng-click="remove(toaster.id)">' +
          '<div ng-class="config.title">{{toaster.title}}</div>' +
          '<div ng-class="config.message">{{toaster.body}}' +
          '</div>' +
          '</div>' +
          '</div>' +
          '</div>'
      };
    }
  ]);


// http://plnkr.co/edit/mVR4P4?p=info

/*
toaster.pop({
  type: 'error',
  title: 'Title text',
  body: 'Body text',
  timeout: 3000
});
toaster.pop({
  type: 'info',
  title: 'Title text',
  body: 'Body text',
  timeout: 3000
});
toaster.pop({
  type: 'success',
  title: 'Title text',
  body: 'Body text',
  timeout: 3000
});
toaster.pop({
  type: 'warning',
  title: 'Title text',
  body: 'Body text',
  timeout: 3000
});


*/