"use strict";

angular.module('siggo')
  .factory('AccountPreferences', function($http, $q, $rootScope, $timeout, configuration,
    localStorageService) {

    var post = {
      method: '',
      url: configuration.http.getUrl() + "driver/account-settings/",
      headers: {
        'Content-Type': 'application/json'
      },
      data: {},
      timeout: 15000
    };


    var _categories;
    var _accessibility;





    var getCategories = function() {
      var _post = angular.copy(post);
      _post.method = 'GET';
      _post.url += 'categories';
      _post.headers.accessToken = $rootScope.Usuario.Token;

      return $http(_post)
        .then(function(res) {
          //console.log(res);
          if (typeof res.data === 'object') {
            _categories = angular.copy(res.data);
            return res.data;
          } else {
            return $q.reject(500);
          }
        }, function(err) {
          if (err.status == 511) { console.log(err); }
          // crear directiva para cuando no está autorizado o redireccionar a loguin ???
          // me agradaria servicio que pase como parametro la ruta actual a loguin
          console.info(err);
          return $q.reject(err.status);
        });

    };


    var putCategories = function(categories) {
      var _post = angular.copy(post);
      _post.method = "PUT";
      _post.url += 'categories';
      _post.data = categories;
      _post.headers.accessToken = $rootScope.Usuario.Token;

      return $http(_post)
        .then(function(res) {
          //console.log(res);
          if (typeof res.data === 'object') {
            //_categories = res.data;
            return res.data;
          } else {
            return $q.reject(500);
          }
        }, function(err) {
          if (err.status == 511) { console.log(err); }
          // crear directiva para cuando no está autorizado o redireccionar a loguin ???
          // me agradaria servicio que pase como parametro la ruta actual a loguin
          console.info(err);
          return $q.reject(err.status);
        });

    };




    var getAccessibility = function() {
      var _post = angular.copy(post);
      _post.method = 'GET';
      _post.url += 'accessibility';
      _post.headers.accessToken = $rootScope.Usuario.Token;

      return $http(_post)
        .then(function(res) {
          //console.log(res);
          if (typeof res.data === 'object') {
            localStorageService.set("account-preferences-accessibility", res.data);
            //_categories = angular.copy(res.data);
            return res.data;
          } else {
            return $q.reject(500);
          }
        }, function(err) {
          if (err.status == 511) { console.log(err); }
          // crear directiva para cuando no está autorizado o redireccionar a loguin ???
          // me agradaria servicio que pase como parametro la ruta actual a loguin
          console.info(err);
          return $q.reject(err.status);
        });

    };


    var putAccessibility = function(data) {
      var _post = angular.copy(post);
      _post.method = "PUT";
      _post.url += 'accessibility';
      _post.data = data;
      _post.headers.accessToken = $rootScope.Usuario.Token;

      return $http(_post)
        .then(function(res) {
          //console.log(res.data);
          if (typeof res.data === 'object') {
            localStorageService.set("account-preferences-accessibility", res.data);
            return res.data;
          } else {
            return $q.reject(500);
          }
        }, function(err) {
          if (err.status == 511) { console.log(err); }
          // crear directiva para cuando no está autorizado o redireccionar a loguin ???
          // me agradaria servicio que pase como parametro la ruta actual a loguin
          console.info(err);
          return $q.reject(err.status);
        });

    };


    // guarantee first request
    var get = function() {
      // wait if there is no token
      if (!$rootScope.Usuario) {
        $timeout(function() {
          get();
        }, 300);
        return;
      }

      getAccessibility()
        .then(function(res) {
          console.log(res);
        }, function(status) {
          $timeout(function() {
            get();
          }, 300);
        });
    }
    get();





    var getPayment = function() {
      var _post = angular.copy(post);
      _post.method = 'GET';
      _post.url += 'payment';
      _post.headers.accessToken = $rootScope.Usuario.Token;

      return $http(_post)
        .then(function(res) {
          //console.log(res);
          if (typeof res.data === 'object') {
            return res.data;
          } else {
            return $q.reject(500);
          }
        }, function(err) {
          if (err.status == 511) { console.log(err); }
          // crear directiva para cuando no está autorizado o redireccionar a loguin ???
          // me agradaria servicio que pase como parametro la ruta actual a loguin
          console.info(err);
          return $q.reject(err.status);
        });

    };


    var putPayment = function(payment) {
      var _post = angular.copy(post);
      _post.method = "PUT";
      _post.url += 'payment';
      _post.data = payment;
      _post.headers.accessToken = $rootScope.Usuario.Token;

      return $http(_post)
        .then(function(res) {
          //console.log(res);
          if (typeof res.data === 'object') {
            return res.data;
          } else {
            return $q.reject(500);
          }
        }, function(err) {
          if (err.status == 511) { console.log(err); }
          // crear directiva para cuando no está autorizado o redireccionar a loguin ???
          // me agradaria servicio que pase como parametro la ruta actual a loguin
          console.info(err);
          return $q.reject(err.status);
        });

    };









    return {
      getCategories: getCategories,
      putCategories: putCategories,
      getAccessibility: getAccessibility,
      putAccessibility: putAccessibility,
      getPayment: getPayment,
      putPayment: putPayment,
      localCategories: function() {
        return angular.copy(_categories);
      },
      localAccessibility: function() {
        var res = localStorageService.get("account-preferences-accessibility");
        return res;
      }
    };
  })
